#include "LPC21xx.h"
#include "uart.h"

unsigned char UART_Received;

void __attribute__ ((interrupt("IRQ"))) uart0ISR(void);

void uart0ISR(void)
{
	unsigned char iid;

	// loop until not more interrupt sources
	while (((iid = U0IIR) & UIIR_NO_INT) == 0)
	{
		// identify & process the highest priority interrupt
    		switch (iid & UIIR_ID_MASK)
      		{
      			case UIIR_RLS_INT:                // Receive Line Status
        			U0LSR;                          // read LSR to clear
        			break;

      			case UIIR_CTI_INT:                // Character Timeout Indicator
      			case UIIR_RDA_INT:                // Receive Data Available
        			do
          			{				//CODE HERE
					UART_Received=U0RBR;
       				}
	       			while (U0LSR & ULSR_RDR);
				break;
      			default:                          // Unknown
        			U0LSR;
        			U0RBR;
        			break;
      		}
    	}

	VICVectAddr = 0x00;
}


void UART_0Init(unsigned short baud, unsigned char mode, unsigned char fmode)
{
	// set port pins for UART0
	PINSEL0 = (PINSEL0 & ~U0_PINMASK) | U0_PINSEL;

	U0IER = 0x00;                         // disable all interrupts
	U0IIR;                                // clear interrupt ID
	U0RBR;                                // clear receive register
	U0LSR;                                // clear line status register

	// set the baudrate
	U0LCR = ULCR_DLAB_ENABLE; 	  // set divisor latches (DLAB)
	U0DLL = (unsigned char)baud;          // set for baud low byte
	U0DLM = (unsigned char)(baud >> 8);   // set for baud high byte

	// set the number of characters and other
	// user specified operating parameters
	U0LCR = (mode & ~ULCR_DLAB_ENABLE); // clear DLAB "on-the-fly"
	U0FCR = fmode;

	// initialize the interrupt vector
	VICIntSelect &= ~BIT(VIC_UART0);  	// UART0 selected as IRQ
	VICIntEnable |= BIT(VIC_UART0);    	// UART0 interrupt enabled
	VICVectCntl1 = VIC_ENABLE | VIC_UART0;
	VICVectAddr1 = (unsigned)uart0ISR;  // address of the ISR

	// enable receiver interrupts
	U0IER = UIER_ERBFI;


}


int UART_0Putch(unsigned char ch)
{
	while (!(U0LSR & ULSR_THRE))	// wait for TX buffer to empty
		continue;                       // also either WDOG() or swap()

	U0THR = (unsigned char)ch;  	// put char to Transmit Holding Register
	return (unsigned char)ch;      
}

int UART_0Getch(void)
{
	if (U0LSR & ULSR_RDR)                 // check if character is available
		return U0RBR;                       // return character

	return -1;
}

int UART_0GetchW(void)
{
	while ( !(U0LSR & ULSR_RDR) ); 		// wait for character 
	return U0RBR;                		// return character
}


const char *UART_0Puts(const char *string)
{
	char ch;
	
	while ((ch = *string)) {
		if (UART_0Putch(ch)<0) break;
		string++;
	}
	
	return string;
}

